@@ -18,11 +18,12 @@
 #
 """PLUSH kind of PyLUceneSHell.
 
-$Id: plush.py 50930 2007-01-30 10:30:36Z bdelbosc $
+$Id: plush.py 51139 2007-02-19 21:22:12Z bdelbosc $
 """
 import sys
 import os
 import re
+from sre_compile import error as RegexError
 from datetime import datetime
 import readline
 from time import time
@@ -38,6 +39,7 @@ from PyLucene import WhitespaceAnalyzer
 from PyLucene import QueryParser, IndexSearcher, FSDirectory
 from PyLucene import IndexReader, Sort, JavaError
 from PyLucene import VERSION, LUCENE_VERSION
+from PyLucene import IndexWriter, Field, Document, RAMDirectory
 
 from version import __version__
 
@@ -48,11 +50,23 @@ def truncStr(text, size=72):
     """Truncate a string."""
     if not text:
         return ''
-    if len(text) < size:
+    if not size or len(text) < size:
         return text
     else:
         return text[:size] + '...'
 
+def guessEncoding():
+    """Guess the cmd encoding."""
+    encoding = 'iso-8859-15'
+    lang = os.getenv('LC_ALL', None)
+    if not lang:
+        lang = os.getenv('LANG')
+    if lang and 'utf-8' in lang.lower():
+        encoding = 'utf-8'
+    return encoding
+
+
+
 class StringReader(object):
     """Class for analyzer input."""
     def __init__(self, u_text):
@@ -99,16 +113,35 @@ class PlushBase:
         self.closeStore()
         self._connected = False
 
+    def initDummyStore(self, directory):
+        """Open a dummy ramdirectory for testing."""
+        writer = IndexWriter(directory, SimpleAnalyzer(), True)
+        doc = Document()
+        doc.add(Field("name", 'dummy.txt', Field.Store.YES,
+                      Field.Index.UN_TOKENIZED))
+        doc.add(Field("path", '/path/to/dummy.txt', Field.Store.YES,
+                      Field.Index.UN_TOKENIZED))
+        doc.add(Field("contents", "foo dummy bar", Field.Store.YES,
+                      Field.Index.TOKENIZED))
+        writer.addDocument(doc)
+        writer.optimize()
+        writer.close()
+
     def openStore(self, store_dir):
         """Open a lucene store."""
         if self._connected:
             self.closeStore()
-        store_path = os.path.abspath(store_dir)
-        try:
-            directory = FSDirectory.getDirectory(store_path, False)
-        except JavaError:
-            print "Error: %s Not found." % store_path
-            return
+        if store_dir == 'dummy':
+            directory = RAMDirectory()
+            self.initDummyStore(directory)
+            store_path = store_dir
+        else:
+            store_path = os.path.abspath(store_dir)
+            try:
+                directory = FSDirectory.getDirectory(store_path, False)
+            except JavaError:
+                print "Error: %s Not found." % store_path
+                return
         try:
             self.searcher = IndexSearcher(directory)
         except JavaError:
@@ -117,7 +150,7 @@ class PlushBase:
         print 'Opening store: %s' % store_path
         self.directory = directory
         self.store_path = store_path
-        self.index_reader = IndexReader.open(self.store_path)
+        self.index_reader = IndexReader.open(directory)
         self.fields = self.getFieldNames()
         self.fields.sort()
         self._connected = True
@@ -197,7 +230,7 @@ class PlushBase:
             return self.searcher.search(query, Sort(sort_on, sort_order))
         return self.searcher.search(query)
 
-    def getTermFreqs(self, field, max_term=None, pattern=None):
+    def getTermFreqs(self, field=None, max_term=None, pattern=None):
         """Return a list of (number of occurence, word) for the field."""
         item = self.index_reader.terms()
         min_freq = 0
@@ -207,10 +240,14 @@ class PlushBase:
         else:
             limit = 1000
         if pattern is not None:
-            pat = re.compile(pattern)
+            try:
+                pat = re.compile(pattern)
+            except RegexError:
+                print "Error: '%s' is an invalid regex" % pattern
+                return []
         while(item.next()):
             term = item.term()
-            if term.field() != field:
+            if field and term.field() != field:
                 continue
             word = term.text()
             freq = item.docFreq()
@@ -236,6 +273,19 @@ class PlushBase:
         self.analyzers['Stop'] = StopAnalyzer()
         self.analyzers['Standard'] = StandardAnalyzer()
         self.analyzers['Whitespace'] = WhitespaceAnalyzer()
+        nxlucene_home = os.getenv('NXLUCENE_HOME', None)
+        if nxlucene_home:
+            # point to http://svn.nuxeo.org/pub/NXLucene/trunk/src/nxlucene
+            nxlucene_home = os.path.normpath(nxlucene_home)
+            sys.path.append(nxlucene_home)
+            try:
+                from analysis import analyzers_map
+            except ImportError:
+                print "Error: Invalid NXLUCENE_HOME %s" % nxlucene_home
+                return
+            for key, value in analyzers_map.items():
+                self.analyzers['nx' + key] = value
+            print "NXLucene analyzers loaded."
 
     def getAnalyzer(self, analyzer_id=None):
         """Return an analyzer or default."""
@@ -266,7 +316,7 @@ Using PyLucene %s and Lucene %s.
         \q or ^D              to quit.""" % (__version__,
                                              VERSION, LUCENE_VERSION)
     plush_attr = ('limit', 'select', 'default_field', 'sort_on',
-                  'waterline', 'analyzer')
+                  'waterline', 'analyzer', 'trunc_size')
     default_field = 'contents'
     select = 'name path'
     limit = 5
@@ -274,11 +324,12 @@ Using PyLucene %s and Lucene %s.
     analyzer = 'Simple'
     history_path = '~/plush_history'
     waterline = ''
+    trunc_size = 150
 
     def __init__(self, store_dir, verbose=False):
         Cmd.__init__(self)
         PlushBase.__init__(self, store_dir, verbose)
-        self.cmd_encoding = self.guessEncoding()
+        self.cmd_encoding = guessEncoding()
         self.prompt = 'plush> '
         self.is_query = True
         self.history_path = os.path.expanduser('~/.plush_history')
@@ -293,16 +344,6 @@ Using PyLucene %s and Lucene %s.
             print "Type: \? for more information."
         return self._connected
 
-    def guessEncoding(self):
-        """Guess the cmd encoding."""
-        encoding = 'iso-8859-15'
-        lang = os.getenv('LC_ALL', None)
-        if not lang:
-            lang = os.getenv('LANG')
-        if lang and 'utf-8' in lang.lower():
-            encoding = 'utf-8'
-        return encoding
-
     def _getResultFieldIds(self):
         """Return list of selected field ids."""
         return filter(None, self.select.split())
@@ -360,6 +401,13 @@ Using PyLucene %s and Lucene %s.
         print "* Number of docs             : %s (max doc num: %s)" % (
             nb_docs, max_doc)
         print "* Number of fields           : %d" % len(self.fields)
+        print "* Number of terms            :",
+        terms = self.index_reader.terms()
+        nb_terms = 0
+        while terms.next():
+            nb_terms += 1
+        terms.close()
+        print nb_terms
         try:
             last_modified = datetime.fromtimestamp(
                 IndexReader.lastModified(directory)/1000.0)
@@ -414,37 +462,57 @@ Using PyLucene %s and Lucene %s.
     do_di = do_describe_indexes
 
     def do_view(self, message):
-        """\\v[iew] [DOC_NUM]
-        View indexes and fields value for [DOC_NUM]
+        """\\v[iew] [DOC_NUM [FIELD_PATTERN]]
+        View values of index fields that matches PATTERN for DOC_NUM
 
         ex:
-        \\v 123"""
+        \\v 123
+        \\v 123 name|path"""
         if not self.checkStore():
             return
         nb_docs = self.maxDoc()
         num_doc = nb_docs - 1
+        pattern = None
         if message:
+            words = message.split(' ', 1)
+            num_doc = None
+            pattern = None
+            if len(words) >= 1:
+                num_doc = words[0].strip()
+            if len(words) == 2:
+                pattern = words[1].strip()
             try:
-                num_doc = int(message)
+                num_doc = int(num_doc)
             except ValueError:
                 print "Error: Invalid value for doc num"
             if num_doc < 0 or num_doc >= nb_docs:
                 print "Error: Invalid doc num"
                 num_doc = nb_docs -1
+        if pattern:
+            try:
+                pat = re.compile(pattern)
+            except RegexError:
+                print "Error: '%s' is an invalid regex" % pattern
+                pattern = None
+
         if self.index_reader.isDeleted(num_doc):
             print "doc num %d is deleted." % num_doc
             return
 
+        trunc_size = self.trunc_size
         fields = self.getFieldInfos(num_doc)
         print "Found %d fields for doc #%d" % (len(fields), num_doc)
         for field in fields:
+            if pattern is not None and not pat.search(field[0]):
+                continue
             print "%-25s  %s %s %s %s %s: %s" % (field[0],
                                                  field[1] and 'S' or ' ',
                                                  field[2] and 'I' or ' ',
                                                  field[3] and 'T' or ' ',
                                                  field[4] and 'B' or ' ',
                                                  field[5] and 'C' or ' ',
-                                                 field[6])
+                                                 truncStr(field[6],
+                                                          trunc_size))
     do_v = do_view
 
     def do_open(self, message):
@@ -498,7 +566,7 @@ Using PyLucene %s and Lucene %s.
         if not value:
             print "  %s = %s" % (key, getattr(self, key, 'unset ?'))
             return
-        if key in ('limit', 'waterline'):
+        if key in ('limit', 'waterline', 'trunc_size'):
             try:
                 value = int(value)
             except ValueError:
@@ -516,7 +584,7 @@ Using PyLucene %s and Lucene %s.
                 return
         if key == 'analyzer':
             if value not in self.analyzers.keys():
-                print "Error: Invalid analyzers."
+                print "Error: Invalid analyzer."
                 print "       Possible values are: %s" % (
                     ', '.join(self.analyzers.keys()),)
                 return
@@ -524,6 +592,14 @@ Using PyLucene %s and Lucene %s.
         print "  %s key setted." % key
         return 0
 
+    def complete_set(self, *args):
+        """Completion for the set cmd."""
+        if 'analyzer' in args[1]:
+            completions = self.analyzers.keys()
+        else:
+            completions = self.plush_attr
+        return [attr for attr in completions if attr.startswith(args[0])]
+
     def do_unset(self, message):
         """\unset [NAME]
         Unset (delete) internal variable."""
@@ -531,6 +607,10 @@ Using PyLucene %s and Lucene %s.
             print "  Unsetted"
             setattr(self, message, '')
 
+    def complete_unset(self, *args):
+        """Completion for the set cmd."""
+        return [attr for attr in self.plush_attr if attr.startswith(args[0])]
+
     def do_nuxset(self, message):
         """Setup default variable for nxlucene."""
         self.default_field = "SearchableText"
@@ -538,12 +618,14 @@ Using PyLucene %s and Lucene %s.
         #self.sort_on = "Title_Sort ASC"
 
     def do_term(self, message):
-        """\\term INDEX_NAME [REGEX]
+        """\\term [INDEX_NAME|*] [REGEX]
         Display top term frequency for INDEX_NAME that match the REGEX.
 
+        term
         term name
         term contents foo
         term contents ^f?o
+        term * foo
         """
         if not self.checkStore():
             return
@@ -553,12 +635,17 @@ Using PyLucene %s and Lucene %s.
             pattern = words[1].strip()
         else:
             pattern = None
-        if field not in self.fields:
+        if pattern and field == '*':
+            field = None
+        if field and field not in self.fields:
             print "No '%s' field in the current index." % field
             return
         print "Top %d term occurences for field '%s' matching '%s'." % (
-            self.limit, field, pattern and pattern or '*')
+            self.limit, field and field or '*', pattern and pattern or '*')
         items = self.getTermFreqs(field, self.limit, pattern)
+        if not items:
+            print "No match."
+            return
         print "occurences term"
         print "---------- ----"
         for freq, term in items:
@@ -619,7 +706,8 @@ Using PyLucene %s and Lucene %s.
             print "* %2.2d) score: %.2f, doc_num: %s" % (count, hits.score(i),
                                                          hits.id(i))
             for field in result_fields:
-                print '  %s: %s' % (field, truncStr(doc.get(field), 150))
+                print '  %s: %s' % (field, truncStr(doc.get(field),
+                                                    self.trunc_size))
             print
             count += 1
         if count:
@@ -630,8 +718,22 @@ Using PyLucene %s and Lucene %s.
             print "%d document with doc_num >= %d" % (count, waterline)
         return 0
 
+
+    def cmdline(self, line):
+        """do cmd line.
+
+        used by doc test."""
+        line = self.precmd(line)
+        stop = self.onecmd(line)
+        stop = self.postcmd(stop, line)
+
+
     # Cmd override to handle backslash cmd ------------------
     doc_header = "Documented commands (type \help <topic>):"
+    def emptyline(self):
+        """Called when an empty line is entered in response to the prompt."""
+        pass
+
     def precmd(self, line):
         """Escaping backslash cmd"""
         if line.startswith('\\'):
